
# Separating Flows in Encrypted Tunnel Traffic
This is the material to reproduce results from the paper [Separating Flows in Encrypted Tunnel Traffic](https://alexhartl.eu/papers/separating.pdf) presented in [ICMLA 2022](https://www.icmla-conference.org/icmla22/).

## Requirements
All experiments have been performed on a Debian buster Linux system and Python 3.8.8. Due to specifics in data generation, we can only support Linux. For dataset generation and preprocessing, [go-flows](https://github.com/CN-TU/go-flows) is needed. For generation of the synthetic data, additionally tcpdump and the [ping++ tool](https://eudl.eu/doi/10.4108/icst.valuetools.2014.258181) are needed.
Install Python requirements using `pip3 -r requirements.txt`. 

## Data Generation

### Synthetic Data
We use py-virtnet for setting up a simulated network and generate test traffic. Scripts for this task are contained in the `synth` directory. Steps to generate a test dataset are as follows:

* Start `vnet2.py` in the `synth` directory:
    ```
    cd synth
    sudo ./vnet2.py
    ```
  `vnet2.py` launches the simulated network and starts helper scripts to generate TCP and UDP test data. `tcpdump` is launched to capture the traffic in the `dump.pcap` file.

* Extract per-packet feature values from `dump.pcap`, creating `dump.csv`:
    ```
    go-flows run features packet.json export csv dump.csv source libpcap dump.pcap
    ```

* Preprocess the data to obtain a pickle file in the `data` directory: 
    ```
    cd ..
    ./parse.py synth/dump.csv data/synth.pickle
    ```

### Real-world Data
To use data from CIC-IDS-2017 or UNSW-NB15, follow these steps:
* Obtain the respective PCAPs from the dataset authors and follow the instructions from the [Datasets-preprocessing](https://github.com/CN-TU/Datasets-preprocessing) repository using the `packet` feature vector to generate a labeled csv with extracted flow information. This will give you a `packet.csv` file for each CIC-IDS-2017 and UNSW-NB15.
* Store the csv in this repository's directory, naming it, e.g., `cicids17.csv` and `unswnb15.csv`, respectively. 
* Preprocess the data to obtain a pickle file in the `data` directory:
    ```
    ./parse.py cicids17.csv data/cicids17.pickle
    ./parse.py unswnb15.csv data/unswnb15.pickle
    ```

## Training
To perform NN training with tensorflow, call
```
./learn.py --mode separate --data <data name> --id <model name>
```
where `<data name>` needs to be replaced with a dataset name used above, e.g., `synth`, `cicids17` or `unswnb15`, and `<model name>` needs to be replaced with a suitable model name. The model will be stored in the `models/<model name>` directory, where models are stored regularly during training.

E.g., to create a model for the `synth` dataset generated above:
```
./learn.py --mode separate --data synth --id synth
```

Call `./learn.py --help` to see further parameters `learn.py` knows to evaluate further variants as described in the paper.

## Evaluation
Having a trained model, separation can be evaluated using `solve.py`. `solve.py` generates interleaved sequences from randomly selected flows and performs separation as outlined in the paper. To evaluate separation, call
```
./solve.py --data <data name> --model <model file name> --interleave-cnt <interleaved flow count>
```

Results will be printed to the screen. To store results, `--result-file` can be specified with a numeric run id. In this case, results will be written to a `.jsonl` file with the corresponding id number within the `results-<data name>` directory. It is advisable to use run ids as specified in `runs.json` with the corresponding parameters, as `runs.json` will be used by the plotting scripts.

E.g., to use the synthetic data generated above with the model `models/synth/100.pickle` and 2 interleaved flows, run
```
./solve.py --data synth --model models/synth/100.pickle --interleave-cnt 2 --result-file 1
```

Call `./solve.py --help` to see further parameters `solve.py` known to evaluate further variants as described in the paper.

## Plotting
We provide our obtained result files in the `results-flowss`, `results-flows`, `results-flows17` and `results-flows15` files. To recreate most plots and results depicted in the paper, use the following calls:
```
./plot.py --data flowss --function acc_plot
./plot.py --data flowss --function maxpos_plot
./plot.py --data flowss --function hist2 --runs 17 19
echo Results for MAWI
./plot.py --data flows --function latex_table --runs 1 2 3 4
echo Results for CIC-IDS-2017
./plot.py --data flows17 --function latex_table --runs 1 2 3 4
echo Results for UNSW-NB15
./plot.py --data flows15 --function latex_table --runs 1 2 3 4
```

Replace the dataset names with your own names to use your own obtained results. Recreating Fig. 6 requires the used dataset for plotting. If you have followed the above steps for creating a synthetic dataset, training and evaluating, you can recreate Fig. 6 using
```
./plot.py --data synth --function steady_plots
```


## Acknowledgments
This work was supported by the project MALware cOmmunication in cRitical Infrastructures (MALORI), funded by the Austrian security research program KIRAS of the Federal Ministry for Agriculture, Regions and Tourism (BMLRT) under grant no. 873511.

