#!/usr/bin/env python3

import asyncio
import numpy as np
import signal

RESPONSE_BYTES = 10*1024

async def handle_client(reader, writer):
    try:
        while True:
            while not b'\n' in await reader.read(1024):
                if reader.at_eof():
                    writer.close() # close() is necessary to immediately close connection and have FIN/ACK in capture
                    return
            tosend = RESPONSE_BYTES
            while tosend > 0:
                random_bytes = np.random.randint(33, 126, min(1024, tosend), np.uint8).tobytes()
                writer.write(random_bytes)
                await writer.drain()
                tosend -= len(random_bytes)
    except OSError:
        pass

async def main():
    loop = asyncio.get_running_loop()
    def shutdown():
        for t in asyncio.all_tasks(loop):
            t.cancel()
    loop.add_signal_handler(signal.SIGINT, shutdown)
    try:
        await (await asyncio.start_server(handle_client, port=10_000)).wait_closed()
    except asyncio.exceptions.CancelledError:
        pass

asyncio.run(main())
