#!/usr/bin/env python3

import json
import sys
import os
import itertools
import signal
import subprocess
import time
import random
from tqdm import tqdm

import numpy as np

import virtnet
from virtnet.container import RouteDirection
from ipaddress import ip_interface
from pyroute2 import IPDB

TOTAL_FLOWS = 100_000

os.setpgid(0, 0)

with virtnet.Manager() as vnet:
    r1 = vnet.Router('r1')
    h1 = vnet.Host('h1')
    h2 = vnet.Host('h2')

    h1.connect(vnet.VirtualLink, r1, 'h1-r1', 'r1-h1')
    h1['h1-r1'].add_ip(ip_interface('10.0.0.2/24'))
    h1['h1-r1'].tc('add', 'netem', rate=1250000000, delay=20000, jitter=2000)
    r1['r1-h1'].add_ip(ip_interface('10.0.0.1/24'))
    h2.connect(vnet.VirtualLink, r1, 'h2-r1', 'r1-h2')
    h2['h2-r1'].add_ip(ip_interface('10.0.1.2/24'))
    h2['h2-r1'].tc('add', 'netem', rate=1250000000, delay=20000, jitter=2000)
    r1['r1-h2'].add_ip(ip_interface('10.0.1.1/24'))

    vnet.simple_route()
    vnet.update_hosts()

    h2.Popen(["tcpdump", "-i", "h2-r1", "-w", "dump.pcap"])
    h2.Popen(["python3", "tcps.py"])
    h2.Popen(["iptables", "-A", "INPUT", "-p", "udp", "--dport", "10001", "-j", "DROP"])
    h2.Popen(["iptables", "-A", "INPUT", "-p", "udp", "--dport", "10002", "-j", "DROP"])
    time.sleep(2)

    clients = []

    def tcp(j):
        clients.append(h1.Popen(["python3", "tcpc.py", "10.0.1.2"]))
    def steady_udp(j):
        subprocess.call(["python3", "udp.py", 'scenarios/%d.csv' % j, 'steady'])
        clients.append(h1.Popen(["ping++", "--protocol", "UDP", "10.0.1.2", "10001", "-z", "scenarios/%d.csv"%j], stdout=subprocess.DEVNULL))
    def bursty_udp(j):
        subprocess.call(["python3", "udp.py", 'scenarios/%d.csv' % j, 'bursty'])
        clients.append(h1.Popen(["ping++", "--protocol", "UDP", "10.0.1.2", "10002", "-z", "scenarios/%d.csv"%j], stdout=subprocess.DEVNULL))

    traffic_generators = [tcp, steady_udp, bursty_udp]

    with tqdm(total=TOTAL_FLOWS) as bar:
        for j in itertools.count():
            time.sleep(.2)
            if j < TOTAL_FLOWS:
                random.choice(traffic_generators)(j)
            elif not clients:
                break
            for i,c in list(enumerate(clients))[::-1]:
                if c.poll() is not None:
                    bar.update(1)
                    # print('Process %d finished' % i)
                    del clients[i]

    signal.signal(signal.SIGINT, signal.SIG_IGN)
    os.killpg(0, signal.SIGINT)